/**
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 2009 Bull S.A.S.
 * Contact: jonas-team@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: BundleListPlugin.java 20762 2011-01-21 13:40:10Z benoitf $
 * --------------------------------------------------------------------------
 */

package org.ow2.jonas.admin.osgi.bundle;

import java.util.ArrayList;

import org.osgi.framework.BundleContext;
import org.osgi.framework.BundleEvent;
import org.osgi.framework.BundleListener;
import org.osgi.framework.ServiceReference;
import org.osgi.service.packageadmin.PackageAdmin;

public class BundleListPlugin {
    private boolean changed;

    public BundleListPlugin(final BundleContext bc) {
        changed = true;
        createEventListener(bc);
    }

    private void createEventListener(final BundleContext bc) {
        BundleListener bl = new BundleListener() {
            public void bundleChanged(BundleEvent event) {
                changed = true;
            }
        };
        bc.addBundleListener(bl);
    }

    public BundleActionResult install(final String url, final BundleContext bc) {
        BundleActionResult bar = new BundleActionResult();
        try {
            bc.installBundle(url, null);
        } catch (Exception ex) {
            bar.setError(true);
            bar.setMessage(ex.getMessage());
        }
        return bar;
    }

    public BundleActionResult action(final String button, final String index, final BundleContext bc) {
        String[] rows = index.split("_");
        boolean error = false;
        String message = "";
        if (button.equals("Start")) {
            for (int i = 0; i < rows.length; i++) {
                try {
                    bc.getBundles()[Integer.parseInt(rows[i])].start();
                } catch (Exception ex) {
                    error = true;
                    message = ex.getMessage();
                }
            }
        } else if (button.equals("Stop")) {
            for (int i = 0; i < rows.length; i++) {
                try {
                    bc.getBundles()[Integer.parseInt(rows[i])].stop();
                } catch (Exception ex) {
                    error = true;
                    message = ex.getMessage();
                }
            }
        } else if (button.equals("Update")) {
            for (int i = 0; i < rows.length; i++) {
                try {
                    bc.getBundles()[Integer.parseInt(rows[i])].update();
                } catch (Exception ex) {
                    error = true;
                    message = ex.getMessage();
                }
            }
        } else if (button.equals("Refresh")) {
            // Get package admin service.
            ServiceReference ref = bc.getServiceReference(PackageAdmin.class.getName());
            if (ref == null) {
                error = true;
                message = "Unable to obtain PackageAdmin service.";
            }

            PackageAdmin pa = (PackageAdmin) bc.getService(ref);
            if (pa == null) {
                error = true;
                message = "Unable to obtain PackageAdmin service.";
            }

            pa.refreshPackages(null);
        } else if (button.equals("Uninstall")) {
            for (int i = rows.length - 1; i >= 0; i--) {
                try {
                    bc.getBundles()[Integer.parseInt(rows[i])].uninstall();
                } catch (Exception ex) {
                    error = true;
                    message = ex.getMessage();
                }
            }
        }
        BundleActionResult bar = new BundleActionResult();
        bar.setError(error);
        bar.setMessage(message);
        return bar;
    }

    public BundleInfoResult create(final BundleContext bc) {
        BundleInfoResult bir = new BundleInfoResult();
        bir.setChanged(true);
        ArrayList<BundleInfo> bundles = new ArrayList<BundleInfo>();
        for (int i = 0; i < bc.getBundles().length; i++) {
            BundleInfo bi = new BundleInfo();
            bi.setId(SimpleTableModel.getValueAt(bc, i, 0).toString());
            bi.setState(SimpleTableModel.getValueAt(bc, i, 1).toString());
            bi.setName(SimpleTableModel.getValueAt(bc, i, 2).toString());
            bi.setSN(SimpleTableModel.getValueAt(bc, i, 3).toString());
            bundles.add(bi);
        }
        bir.setBundlelist(bundles);
        changed = false;
        return bir;
    }

    public BundleInfoResult update(final BundleContext bc) {
        BundleInfoResult bir = new BundleInfoResult();
        while (!changed) {
            try {
                Thread.sleep(1000);
            } catch (InterruptedException e) {
                e.printStackTrace();
            }
        }
        if (changed) {
            bir.setChanged(true);
            ArrayList<BundleInfo> bundles = new ArrayList<BundleInfo>();
            for (int i = 0; i < bc.getBundles().length; i++) {
                BundleInfo bi = new BundleInfo();
                bi.setId(SimpleTableModel.getValueAt(bc, i, 0).toString());
                bi.setState(SimpleTableModel.getValueAt(bc, i, 1).toString());
                bi.setName(SimpleTableModel.getValueAt(bc, i, 2).toString());
                bi.setSN(SimpleTableModel.getValueAt(bc, i, 3).toString());
                bundles.add(bi);
            }
            bir.setBundlelist(bundles);
            changed = false;
        } else {
            bir.setChanged(false);
        }
        return bir;
    }
}
